// Picture.cpp : implementation file
//

#include "stdafx.h"
#include "Picture.h"
#include <objbase.h>
#include "enh.h"

// CPicture

IMPLEMENT_DYNAMIC(CPicture, CStatic)


/****************************************************************************
*                             CPicture::CPicture
* Effect: 
*       Constructor
****************************************************************************/

CPicture::CPicture()
{
 bkcolor = RGB(0,0,0);
}


/****************************************************************************
*                            CPicture::~CPicture
*       
* Effect: 
*       Destructor
****************************************************************************/

CPicture::~CPicture()
{
}


/****************************************************************************
*                                 Message Map
****************************************************************************/

BEGIN_MESSAGE_MAP(CPicture, CStatic)
    ON_WM_PAINT()
END_MESSAGE_MAP()



// CPicture message handlers


/****************************************************************************
*                              CPicture::OnPaint
* Result: void
*       
* Effect: 
*       Draws the bitmap
****************************************************************************/

void CPicture::OnPaint()
    {
     CPaintDC dc(this); // device context for painting

     CRect r;
     GetClientRect(&r);
     dc.FillSolidRect(&r, bkcolor);

     if(!image.IsNull())
        { /* draw bitmap */
         CRgn rgn;
         int save = dc.SaveDC();
         rgn.CreateRectRgnIndirect(&r);
         dc.SelectClipRgn(&rgn);
         image.BitBlt(dc, 0, 0, image.GetWidth(), image.GetHeight(), 0, 0, SRCCOPY);
         dc.RestoreDC(save);
        } /* draw bitmap */
    }

/****************************************************************************
*                               CPicture::Load
* Inputs:
*       const CString & filename: File name to load
* Result: HRESULT
*       Return code, SUCCEEDED(HRESULT) if success
* Effect: 
*       Loads the image from the file
****************************************************************************/

HRESULT CPicture::Load(const CString & filename)
    {
     if(!image.IsNull())
         image.Destroy();

     HRESULT result = image.Load(filename);     
     if(FAILED(result))
        return result;

     Invalidate();
     return result;
    } // CPicture::Load

/****************************************************************************
*                               CPicture::Save
* Inputs:
*       const CString & filename:
*       REFGUID type = GUID_NULL: The file type (GUID_NULL will use ImageFormatBMP)
* Result: HRESULT
*       SUCCEEDED(HRESULT) if success, otherwise error code
* Effect: 
*       Saves the image
****************************************************************************/

HRESULT CPicture::Save(const CString & filename, REFGUID type /* = GUID_NULL */)
    {
     REFGUID fmt = (type == GUID_NULL ? Gdiplus::ImageFormatBMP : type);

     HRESULT result = image.Save(filename, fmt);
     return result;
    } // CPicture::Save

/****************************************************************************
*                            CPicture::MakeTarget
* Inputs:
*       int cx: Bitmap width
*       int cy: Bitmap height
* Result: BOOL
*       TRUE if successful
* Effect: 
*       Creates a target bitmap
****************************************************************************/

BOOL CPicture::MakeTarget(int cx, int cy)
    {
     if(!image.IsNull())
        { /* see if already there */
         if(image.GetWidth() == cx &&
            image.GetHeight() == cy)
            return TRUE; // already exists
         image.Destroy(); // get rid of wrong-sized image
        } /* see if already there */

     if(!image.Create(cx, cy, 24))
        return FALSE;
     Invalidate();
     return TRUE;
    } // CPicture::MakeTarget

/****************************************************************************
*                            CPicture::MakeTarget
* Inputs:
*       const CPicture & source
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Creates a bitmap big enough to hold source
****************************************************************************/

BOOL CPicture::MakeTarget(const CPicture & source)
    {
     return MakeTarget(source.image.GetWidth(), source.image.GetHeight());
    } // CPicture::MakeTarget

/****************************************************************************
*                             CPicture::Transfer
* Inputs:
*       CPicture & source
*       UINT mode: Type of transfer
* Result: void
*       
* Effect: 
*       Transfers the source picture here
****************************************************************************/

void CPicture::Transfer(const CPicture & source, UINT mode)
    {
     if(!MakeTarget(source))
        return;

     HDC dc = image.GetDC();
     UINT rop = mode & ~ENH_FLAG;
     source.image.BitBlt(dc, 0, 0, source.image.GetWidth(), source.image.GetHeight(), 0, 0, mode);

     if(mode & ENH_FLAG)
        Enhance();
     image.ReleaseDC();
     Invalidate();
    } // CPicture::Transfer

/****************************************************************************
*                              CPicture::Enhance
* Result: void
*       
* Effect: 
*       Enhances low-threshold changes  
****************************************************************************/

void CPicture::Enhance()
    {
     for(int x = 0; x < image.GetWidth(); x++)
        for(int y = 0; y < image.GetHeight(); y++)
           { /* scan image */
            COLORREF color = image.GetPixel(x, y);
#define ENHANCEVALUE(x) (((x) == 0) ? 0 : (((x) < 80) ? ((x) + 80) : (x)))
            COLORREF enhanced = RGB(
                                    ENHANCEVALUE(GetRValue(color)),
                                    ENHANCEVALUE(GetGValue(color)),
                                    ENHANCEVALUE(GetBValue(color)));
            image.SetPixel(x, y, enhanced);
           } /* scan image */
    } // CPicture::Enhance
